/**
 * Tables, primary keys, sequences & Indexes
 */

--IMPORTANT: Postgres sequence naming schema for SERIAL columns: <table name>_<column name>_seq.
--IMPORTANT: This sequence will automatically get dropped when SAS_BDN_DGROLS gets dropped since it is owned by SAS_BDN_DGROLS

--IMPORTANT: due to bug in 3.1 this better way (pg_catalog.setval(pg_get_serial_sequence('SAS_BDN_DGROLS', 'dg_id'), 1000000)) is not possible
--IMPORTANT: 2nd param (column name is Case-SenTiVe) must be 'dg_id', but 1st is NOT or so it appears. VERY unexpected! Should be using macros
--select pg_catalog.setval(pg_get_serial_sequence('SAS_BDN_DGROLs', 'dg_id'), 1000000);

create table SAS_BDN_DGVERS
(
    DG_VERS varchar(16) not null,
    DG_DBID varchar(8),
    DG_CRDT timestamp with time zone not null,
    DG_UPDT timestamp with time zone not null
);

create table SAS_BDN_DGTAGS
(
	DG_ID 	SERIAL 		not null,
	DG_NAME varchar(50) not null,
	constraint SAS_BDN_DGTAGS_PK primary key (DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGTAGS_DG_ID_SEQ RESTART WITH 1000100;
create index SAS_BDN_DGTAGS_NAME_IDX on SAS_BDN_DGTAGS
(
	DG_NAME
);

create table SAS_BDN_DGROLS
(
	DG_ID 		SERIAL 		not null,
	DG_NAME varchar(50) not null,
	DGS_DESC text,
	DG_TYPE varchar(1) not null,
	DG_TTID INTEGER NOT NULL,
	constraint SAS_BDN_DGROLS_PK primary key (DG_ID),
	constraint SAS_BDN_DGROLS_UK unique (DG_TTID, DG_NAME)
);
ALTER SEQUENCE SAS_BDN_DGROLS_DG_ID_SEQ RESTART WITH 1000200;

create table SAS_BDN_DGATRC
(
	DG_ID 		SERIAL 		not null,
	DG_TYPE varchar(50) not null,
	DGS_DESC text,
	constraint SAS_BDN_DGATRC_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGATRC_DG_ID_SEQ RESTART WITH 1000400;

create table SAS_BDN_DGATTY
(
	DG_ID 		SERIAL 		not null,
	DG_TYPE varchar(50) not null,
	DGS_DESC text,
	constraint SAS_BDN_DGATTY_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGATTY_DG_ID_SEQ RESTART WITH 1000500;

create table SAS_BDN_DGATRS
(
	DG_ID 		SERIAL 			not null,
	DG_ATID integer not null,
	DG_CTID integer not null,
	DG_NAME varchar(255) not null,
	DGS_DESC text,
	DG_TYPE varchar(1),
	DG_ISRQ varchar(1),
	DG_CRDT timestamp not null,
	constraint SAS_BDN_DGATRS_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGATRS_DG_ID_SEQ RESTART WITH 1000600;

create table SAS_BDN_DGITAV
(
	DG_ID 	SERIAL 	not null,
	DGS_VAL text,
	constraint SAS_BDN_DGITAV_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGITAV_DG_ID_SEQ RESTART WITH 1000800;

create table SAS_BDN_DGOBJS
(
	DG_ID 		SERIAL 		not null,
	DG_PID integer,
	DG_RTID integer,
	DG_TTID integer,
	DG_NAME varchar(100) not null,
	DGS_DESC text,
	DG_TYPE varchar(1) not null,
	DG_EN varchar(1) not null,
	DG_ACTV varchar(1) not null,
	DG_SCP varchar(1) not null,
	DG_LOCK varchar(1) not null,
	DG_WFID varchar(100),
	DG_PEND varchar(1) not null,
	constraint SAS_BDN_DGOBJS_PK primary key(DG_ID),
    constraint SAS_BDN_DGOBJS_UK unique(DG_RTID)
);
ALTER SEQUENCE SAS_BDN_DGOBJS_DG_ID_SEQ RESTART WITH 1000900;
create index SAS_BDN_DGOBJS_MC_IDX1 on SAS_BDN_DGOBJS 
(
	DG_EN ASC,
	DG_ID ASC
);

create index SAS_BDN_DGOBJS_TTID_IDX on SAS_BDN_DGOBJS 
(
	DG_TTID ASC
);

create table SAS_BDN_DGOVR
(
	DG_ID 	SERIAL 		not null,
	DG_OID integer not null,
	DG_VER integer not null,
	DG_MVER integer not null,
	DG_LBL varchar(200),
	DG_EN varchar(1) not null,
	DG_ACTV varchar(1) not null,
	DG_SCP varchar(1) not null,
	DG_UID varchar(50) not null,
	DG_ISLK varchar(1) not null,
	DG_LKID varchar(50),
	DG_LKDT timestamp,
	DG_CRDT timestamp not null,
	constraint SAS_BDN_DGOVR_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGOVR_DG_ID_SEQ RESTART WITH 1001000;
create index SAS_BDN_DGOVR_MC_IDX1 on SAS_BDN_DGOVR
(
	DG_OID ASC,
	DG_VER ASC
);
create index SAS_BDN_DGOVR_MC_IDX2 on SAS_BDN_DGOVR
(
	DG_OID ASC,
	DG_EN ASC,
	DG_VER ASC
);
create index SAS_BDN_DGOVR_MC_IDX3 on SAS_BDN_DGOVR
(
	DG_VER ASC,
	DG_OID ASC,
	DG_ID ASC,
	DG_EN ASC
);

create table SAS_BDN_DGOTGS
(
	DG_OID integer not null,
	DG_TID integer not null,
	constraint SAS_BDN_DGOTGS_PK primary key(DG_OID, DG_TID)
);

create table SAS_BDN_DGOAS
(
	DG_OID integer not null,
	DG_ROID integer not null,
	DG_NAME varchar(50),
	constraint SAS_BDN_DGOAS_PK primary key(DG_OID,DG_ROID)
);

create table SAS_BDN_DGOACC
(
	DG_OID integer not null,
	DG_UID varchar(50) not null,
	DG_RID integer not null,
	constraint SAS_BDN_DGOACC_PK primary key(DG_OID,DG_UID,DG_RID)
);

create table SAS_BDN_DGNOTS
(
	DG_ID 		SERIAL 		not null,
	DG_OID integer,
	DG_TYPE varchar(1),
	DG_TKID integer,
	DG_TNM varchar(30),
	DGS_TEXT text not null,
    DG_UID varchar(50) not null,
	DG_CRDT timestamp not null,
	constraint SAS_BDN_DGNOTS_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGNOTS_DG_ID_SEQ RESTART WITH 1001200;
create index SAS_BDN_DGNOTS_MC_IDX1 on SAS_BDN_DGNOTS
(
	DG_OID ASC,
	DG_TYPE ASC
);

create table SAS_BDN_DGIT
(
	DG_ID 	SERIAL 		not null,
	DG_OID integer,
	DG_PID integer not null,
	DG_IK varchar(50),
	DG_IV varchar(50),
	DG_LI varchar(50),
	DG_EN varchar(1) not null,
	DG_ACTV varchar(1) not null,
	DG_SCP varchar(1) not null,
	DG_UID varchar(50) not null,
	constraint SAS_BDN_DGIT_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGIT_DG_ID_SEQ RESTART WITH 1001300;
create index SAS_BDN_DGIT_MC_IDX1 on SAS_BDN_DGIT
(
	DG_OID ASC,
	DG_PID ASC,
	DG_IK ASC,
	DG_ID ASC
);
create index SAS_BDN_DGIT_MC_IDX2 on SAS_BDN_DGIT
(
	DG_PID ASC,
	DG_IK ASC,
	DG_ID ASC
);

create table SAS_BDN_DGITVR
(
	DG_OVID integer not null,
	DG_IID integer not null,
	constraint SAS_BDN_DGITVR_PK primary key(DG_OVID, DG_IID)
);

create table SAS_BDN_DGIA
(
	DG_ID 	SERIAL 	not null,
	DG_IID integer not null,
	DG_AID integer not null,
	constraint SAS_BDN_DGIA_PK primary key(DG_ID)
);
ALTER SEQUENCE SAS_BDN_DGIA_DG_ID_SEQ RESTART WITH 1001500;
create index SAS_BDN_DGIA_MC_IDX1 on SAS_BDN_DGIA
(
	DG_AID ASC,
	DG_ID ASC,
	DG_IID ASC
);
create index SAS_BDN_DGIA_MC_IDX2 on SAS_BDN_DGIA
(
	DG_IID ASC,
	DG_AID ASC
);

create table SAS_BDN_DGIAVV
(
	DG_OVID integer not null,
	DG_IAVID integer not null,
	DG_IAID integer not null,
	constraint SAS_BDN_DGIAVV_PK primary key(DG_OVID, DG_IAVID, DG_IAID)
);
create index SAS_BDN_DGIAVV_MC_IDX1 on SAS_BDN_DGIAVV
(
	DG_OVID ASC,
	DG_IAID ASC,
	DG_IAVID ASC
);

create table SAS_BDN_DGIAVR
(
	DG_OVID integer not null,
	DG_IAID integer not null,
	constraint SAS_BDN_DGIAVR_PK primary key(DG_OVID, DG_IAID)
);

/**
 * Foreign Keys
 */
alter table SAS_BDN_DGATRS add constraint SAS_BDN_FKDGATRS_DGATRC
	foreign key(DG_CTID)
	references SAS_BDN_DGATRC (DG_ID);

alter table SAS_BDN_DGATRS add constraint SAS_BDN_FKDGATRS_DGATTY
	foreign key(DG_ATID)
	references SAS_BDN_DGATTY (DG_ID);
	
alter table SAS_BDN_DGIA add constraint SAS_BDN_FKDGIA_DGATRS
	foreign key(DG_AID)
	references SAS_BDN_DGATRS (DG_ID);

alter table SAS_BDN_DGIA add constraint SAS_BDN_FKDGIA_DGIT
	foreign key(DG_IID)
	references SAS_BDN_DGIT (DG_ID);

alter table SAS_BDN_DGIAVR add constraint SAS_BDN_FKDGIAVR_DGIA
	foreign key(DG_IAID)
	references SAS_BDN_DGIA (DG_ID);

alter table SAS_BDN_DGIAVR add constraint SAS_BDN_FKDGIAVR_DGOVR
	foreign key(DG_OVID)
	references SAS_BDN_DGOVR (DG_ID);

alter table SAS_BDN_DGIAVV add constraint SAS_BDN_FKDGIAVV_DGIA
	foreign key(DG_IAID)
	references SAS_BDN_DGIA (DG_ID);

alter table SAS_BDN_DGIAVV add constraint SAS_BDN_FKDGIAVV_DGITAV
	foreign key(DG_IAVID)
	references SAS_BDN_DGITAV (DG_ID);

alter table SAS_BDN_DGIAVV add constraint SAS_BDN_FKDGIAVV_DGOVR
	foreign key(DG_OVID)
	references SAS_BDN_DGOVR (DG_ID);

alter table SAS_BDN_DGIT add constraint SAS_BDN_FKDGIT_DGOBJS
	foreign key(DG_OID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGITVR add constraint SAS_BDN_FKDGITVR_DGIT
	foreign key(DG_IID)
	references SAS_BDN_DGIT (DG_ID);

alter table SAS_BDN_DGITVR add constraint SAS_BDN_FKDGITVR_DGOVR
	foreign key(DG_OVID)
	references SAS_BDN_DGOVR (DG_ID);

alter table SAS_BDN_DGNOTS add constraint SAS_BDN_FKDGNOTS_DGOBJS
	foreign key(DG_OID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGOACC add constraint SAS_BDN_FKDGOACC_DGOBJS
	foreign key(DG_OID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGOACC add constraint SAS_BDN_FKDGOACC_DGROLS
	foreign key(DG_RID)
	references SAS_BDN_DGROLS (DG_ID);

alter table SAS_BDN_DGOAS add constraint SAS_BDN_FKDGOAS_DGOBJS1
	foreign key(DG_OID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGOAS add constraint SAS_BDN_FKDGOAS_DGOBJS2
	foreign key(DG_ROID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGOBJS add constraint SAS_BDN_FKDGOBJS_PID
	foreign key(DG_PID)
	references SAS_BDN_DGOBJS (DG_ID);
	
alter table SAS_BDN_DGOBJS add constraint SAS_BDN_FKDGOBJS_TTID
	foreign key(DG_TTID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGOTGS add constraint SAS_BDN_FKDGOTGS_DGOBJS
	foreign key(DG_OID)
	references SAS_BDN_DGOBJS (DG_ID);

alter table SAS_BDN_DGOTGS add constraint SAS_BDN_FKDGOTGS_DGTAGS
	foreign key(DG_TID)
	references SAS_BDN_DGTAGS (DG_ID);

alter table SAS_BDN_DGOVR add constraint SAS_BDN_FKDGOVR_DGOBJS
	foreign key(DG_OID)
	references SAS_BDN_DGOBJS (DG_ID);
	
alter table SAS_BDN_DGROLS add constraint SAS_BDN_FKDGROLS_DGOBJS
	foreign key(DG_TTID)
	references SAS_BDN_DGOBJS (DG_ID);

