<%@ taglib prefix="form" uri="http://www.springframework.org/tags/form" %>

<%@ include file="/WEB-INF/jsp/header.jsp" %>

<script>
function handleHostChanged()
{
    var hostField = document.getElementById("host");
    var host = hostField.value.trim();
    if (host.length == 0)
    {
        alert("Database server host must not be blank.")
        hostField.focus();
    }
    else
        updateUrl();
}

function handlePortChanged()
{
    var portField = document.getElementById("port");
    var port = portField.value.trim();
    if ((port.length == 0) || isValidPort(port))
        updateUrl();
    else
    {
        alert(port + " is an invalid value for port.")
        portField.focus();
    }
}

function handleDatabaseChanged()
{
    var databaseField = document.getElementById("database");
    var database = databaseField.value.trim();
    if (database.length == 0)
    {
        alert("Database name must not be blank.");
        databaseField.focus();
    }
    else
        updateUrl();
}

function updateUrl()
{
    if (document.getElementById("overrideUrl").checked)
        return;

    var type = document.getElementById("type").value.trim();
    var host = document.getElementById("host").value.trim();
    var port = document.getElementById("port").value.trim();
    var database = document.getElementById("database").value.trim();

    var root;
    var separator;
    if ("SQL Server" == type)
    {
        root = "jdbc:sqlserver://";
        separator = ";database=";
    }
    else if ("Oracle" == type)
    {
        root = "jdbc:oracle:thin:@";
        separator = ":"
    }
    else
    {
        root = "jdbc:db2://";
        separator = "/"
    }

    var url = root + host;
    if ((port.length != 0) && isValidPort(port))
            url += ":" + port;
    url += separator + database 
    document.getElementById("databaseUrl").value = url;
}

function isValidPort(str) 
{
    // 0 is valid
    // 1-9 followed by any number of digits is valid
    return /^(0|[1-9]\d*)$/.test(str);
}

function handleOverride(value)
{
    document.getElementById("host").readOnly = value;
    document.getElementById("port").readOnly = value;
    document.getElementById("database").readOnly = value;
    document.getElementById("databaseUrl").readOnly = !value
    updateUrl();
}
</script>

<form:form modelAttribute="migrationConfiguration" action="${pageContext.request.contextPath}/migration/configure" method="post">
    <h1>Business Data Network Migration</h1>
    <p/>
    <h3>Connection to BDN 2.x database server</h3>
    <table>
        <tr id="typeRow">
            <td>Type of database server:</td>
            <td>
                <form:select path="type" id="type" onChange="updateUrl()">
                    <form:option value="SQL Server"/>
                    <form:option value="Oracle"/>
                    <form:option value="DB2"/>
                </form:select>
            </td>
        </tr>
        <tr id="hostRow">
            <td>Database server host:</td>
            <td><form:input path="host" id="host" onChange="handleHostChanged()"/></td>
        </tr>
        <tr id="portRow">
            <td>Database server port:</td>
            <td><form:input path="port" id="port" onChange="handlePortChanged()"/></td>
        </tr>
        <tr id="databaseRow">
            <td>Database name:</td>
            <td><form:input path="database" id="database" onChange="handleDatabaseChanged()"/></td>
        </tr>
        <tr>
            <td><form:checkbox path="overrideUrl" id="overrideUrl" onclick="handleOverride(this.checked)"/>Override generated JDBC URL</td>
        </tr>
        <tr>
            <td>JDBC URL to database:</td>
            <td><form:input path="url" readOnly="true" size="60" id="databaseUrl"/></td>
        </tr>
        <tr>
            <td>BDN 2.x database connection properties:</td>
            <td><form:input path="properties" size="60"/></td>
        </tr>
        <tr>
            <td>Optional prefix for BDN 2.x tables:</td>
            <td><form:input path="prefix" /></td>
        </tr>
        <tr>
            <td>User name for database connection:</td>
            <td><form:input path="username"/></td>
        </tr>
        <tr>
            <td>Password for database connection:</td>
            <td><form:password path="password"/></td>
        </tr>
    </table>
    <p/>
    <hr/>

    <h3>Options</h3>
    <form:checkbox path="keepTempTables"/>Keep the temporary tables<br/>
    <form:checkbox path="skipCopySourceTables"/>Skip copying the tables from the source BDN database<br/>
    <em>If you have run a migration before and kept the temporary tables, skipping the copy of the source tables will make the migration a bit faster.</em><br/>
    <form:checkbox path="readOnly"/>Test run (run migration without make any changes)<br/>
    <hr/></p>

    <input id="save" type="submit" value="RUN" /></br>
    <p style="color:red;">${migrationConfiguration.error}</p>
  </p>
</form:form>

<script>
handleOverride(document.getElementById("overrideUrl").checked); // causes updateUrl to be called
</script>

<%@ include file="/WEB-INF/jsp/footer.jsp" %>
