<%@ page language="java" contentType="text/html; charset=UTF-8" pageEncoding="utf-8"%>
<%@ page trimDirectiveWhitespaces="true" %>
<%@ taglib prefix="c" uri="http://java.sun.com/jsp/jstl/core" %>
<%@ taglib prefix="fn" uri="http://java.sun.com/jsp/jstl/functions"  %>
<%@ taglib prefix="spring" uri="http://www.springframework.org/tags" %>
<%@ taglib prefix="dmvSearch" tagdir="/WEB-INF/tags/search" %>
<!DOCTYPE html>
<html>
<head>
    <dmvSearch:commonHeadResources />
</head>
<body class="claro">
	<dmvSearch:header />
    <div class="content">
    	<div class="resultHeader">
    		<c:url value="${loginModel.backToSearchURL}" var="backToSearchURLTag" />
    		<spring:message code="label.BACK_TO_SEARCH" var="backToSearchLabel" />
    		<p class="backToSearch"><a href="${backToSearchURLTag}" title="${backToSearchLabel}" >${backToSearchLabel}</a></p>
    		<div class="hr" ></div>
    	</div>
    	<dmvSearch:noscript />
    	<noscript>
    		<!-- wrap the page content in a div with the style set to display:none rather than having javascript
    			 set the style on the content itself to work around the issue in IE where the enter button does not
    			 submit the form properly if the form is not displayed when the page loads -->
    		<div style="display:none">
    	</noscript>
    	<c:if test="${loginModel.sameCredentialsSelected}">
    		<c:set value="checked" var="shareCredentialsSelectedValue" />
    		<c:set value="" var="uniqueCredentialsSelectedValue" />
    	</c:if>
    	<c:if test="${loginModel.uniqueCredentialsSelected}">
    		<c:set value="" var="shareCredentialsSelectedValue" />
    		<c:set value="checked" var="uniqueCredentialsSelectedValue" />
    	</c:if>
    	<spring:message code="value.DUMMY_PASSWORD" var="dummyPassword" />
    	<c:choose>
    		<c:when test="${loginModel.hasCredentials}">
    			
    			<c:choose>
    				<c:when test="${fn:length(loginModel.credentialSet.masterCredentials.key) > 0 }">
    					<c:set value=' value="${loginModel.credentialSet.masterCredentials.key}"' var="masterUsername" />
    				</c:when>
    				<c:otherwise>
    					<c:set value="" var="masterUsername" />
    				</c:otherwise>
    			</c:choose>
    			
    			<c:choose>
    				<c:when test="${fn:length(loginModel.credentialSet.masterCredentials.value) > 0 }">
    					<c:set value=' value="${dummyPassword}"' var="masterPassword" />
    				</c:when>
    				<c:otherwise>
    					<c:set value="" var="masterPassword" />
    				</c:otherwise>
    			</c:choose>
    			
    			<c:choose>
    				<c:when test="${loginModel.partialAuthenticationFailure}">
    					<%-- partial sign in --%>
    					<c:set var="signedInStatusLabel">
    						<p>${loginModel.signedInMessage}</p>
    						<p>${loginModel.notSignedInMessage}</p>
    					</c:set>
    				</c:when>
    				<c:otherwise>
    					<%-- complete successful sign in --%>
    					<c:set var="signedInStatusLabel">
    						<p><spring:message code="label.SIGNED_IN" /></p>
    					</c:set>
    				</c:otherwise>
    			</c:choose>
    		</c:when>
    		<c:otherwise>
    			<c:set var="signedInStatusLabel">
	    			<p><spring:message code="label.NOT_SIGNED_IN_MESSAGE" /></p>
    			</c:set>
    			<c:set value="" var="masterUsername" />
    			<c:set value="" var="masterPassword" />
    		</c:otherwise>
    	</c:choose>
    	<div class="logInFormContainer">
    		<h3><spring:message code="header.SIGN_IN" /></h3>
    		${signedInStatusLabel}
    		<p><spring:message code="label.SPECIFY_CREDENTIALS" arguments="${loginModel.providerNames}" argumentSeparator="|" /></p>
    		<c:url value="/search/j_spring_security_check" var="url" />
    		<form id="loginForm" action="${url}" method="POST">
	        <c:if test="${loginModel.showMultipleSearchProviderOptions}">
	        	<input type="radio" name="credentialSelect" value="share" id="useSameCredentials" ${shareCredentialsSelectedValue} onclick="dmv.search.vars.controller.onCredentialSelectChange(this.form[this.name]);"/>
	        	<label for="useSameCredentials"><spring:message code="radio.option.label.SAME_CREDENTIALS" /></label>
	        </c:if>
	        <div class="formEntry">
	        	<label for="username"><spring:message code="label.USER_ID" /></label>
	        	<input id="username" name="j_username" type="text" ${masterUsername} onfocus="this.select();" maxlength="64" />
			</div>
			<div class="formEntry">
	        	<label for="password"><spring:message code="label.PASSWORD" /></label>
	        	<input id="password" name="j_password" type="password" ${masterPassword} onfocus="this.select();" maxlength="64" />
			</div>
			<c:if test="${loginModel.showMultipleSearchProviderOptions}">
				<input type="radio" name="credentialSelect" value="unique" id="useDifferentCredentials" ${uniqueCredentialsSelectedValue} onclick="dmv.search.vars.controller.onCredentialSelectChange(this.form[this.name]);" />
				<label for="useDifferentCredentials"><spring:message code="radio.option.label.DIFFERENT_CREDENTIALS" /></label>
				<c:forEach items="${registeredSearchProviders}" var="searchProvider">
					<c:set var="spUsername" value="" />
					<c:set var="spPassword" value="" />
					<c:if test="${loginModel.credentialSet.overriddenCredentials[searchProvider.ID] != null }">
						<c:set var="spUsername" value="${loginModel.credentialSet.overriddenCredentials[searchProvider.ID].key}" />
						<c:set var="spPassword" value="${dummyPassword}" />
					</c:if>
					<dmvSearch:searchProviderCredentials searchProvider="${searchProvider}" username="${spUsername}" password="${spPassword}" />
				</c:forEach>
			</c:if>
			<spring:message code="label.SIGN_IN" var="submitLabel" />
			<br />
			<c:url value="${loginModel.redirectURL}" var="redirectURLTag" />
			<input type="hidden" value="${redirectURLTag}" name="successDest" />
			<input type="submit" value="${submitLabel}" class="inputButton" style="margin-top:10px;" />
			</form>
			<c:url value="${initParam.searchResourcesPathPrefix}/images/StatusError12.png" var="errorImgURL" />
			<spring:message code="img.alt.ERROR" var="errorImgAlt" />
			<%-- only show the error if all of them have failed to authenticate --%>
			<c:if test="${loginModel.totalAuthenticationFailure}">
				<c:forEach items="${loginModel.errorMessages}" var="errorMessage">
					<div>
						<img src="${errorImgURL}" alt="${errorImgAlt}" title="${errorImgAlt}" class="signInErrorImg" />
						<p class="signInErrorMsg">${errorMessage}</p>
					</div>
				</c:forEach>
			</c:if>
    	</div>
    </div>
    <noscript>
    		<!-- end workaround -->
    		</div>
    </noscript>
    <c:url var="controllerjsURL" value="${initParam.searchResourcesPathPrefix}/script/dmv/loginController.js" />
	<script>
           require(["dojo/ready","dojo/io/script","dojo/_base/connect", "dojo/query","dojo/dom-style"], 
                    function(ready, script, connect, $, domStyle) {
        	   			ready(function(){
        	   				// the loginController is going to publish a loginControllerReady topic when it's 
        	   				// finished initializing.  Listen for it here.  If this check isn't made the call to 
        	   				// create the loginController object fails becasue it hasn't finished initializing 
        	   				// even though it's done loading.
        	   				var handle = connect.subscribe("loginControllerReady", function(){
        	   					console.log("handled: loginControllerReady");
	                        	dmv.search.vars.controller = new dmv.search.loginController();
	                            dmv.search.vars.controller.init();
                            	connect.unsubscribe(handle);
        	   				});
	 	   					script.get({
		                        url:"${controllerjsURL}",
		                        load: function(evt){
		                        	console.log("loaded");
		                        }
	                    	 });
        	   			});
           			}
           );
    </script>
</body>
</html>