/**
 * The bad thing about declare() is that everything is public, which means it's important to stick
 * to the convention of putting an underscore in front of "private" declarations
 */
require(["dojo/_base/declare",
         "dojo/query",
         "dojo/_base/connect",
         "dojo/dom-style",
         "dojo/dom-attr",
         "dojo/_base/fx",
         "dojo/i18n!i18n/nls/strings",
         "dojo/_base/array",
         "dojo/_base/window",
         "dojo/has",
         "dojo/dom-geometry"], 
		function(declare,
				 $,
				 connect,
				 domStyle,
				 domAttr,
				 fx,
				 resources,
				 arrayFunctions,
				 win,
				 has,
				 domGeom){
  declare("dmv.search.searchController", null, {
	
    constructor: function(/*Object*/ searchProviderProperties){
    	this._searchProviderProperties = searchProviderProperties;
    	/*Object[String, Handle]*/
    	this._handles = {};
		/*Array*/
    	this._selectedSearchProviders = [];
    	this._bindSetter(this._onAfterSearchHandler, this, "onSearchProviderSelect");
    },
    CONST:{_toggledClassSuffix:"__toggled"},
    
    getHandle: function(handleName){
		return this._handles[handleName];
	},
	/**
	 * @param {String} handleName
	 * @param {Handle} value
	 * @return {Boolean} return true if the handle was created, false if the value was overridden
	 */
	setHandle: function(/*String*/ handleName, /*Handle*/ value){
		var handleAlreadyExists = this._handles[handleName] == true;
		this._handles[handleName] = value;
		return handleAlreadyExists;
	},
	
	getSearchProviders:function(){
		// translate the css class names into something more appropriate for the server to use
		var searchProviderLen = this._selectedSearchProviders.length;
		var translatedSearchProviderArray = new Array(searchProviderLen);
		for(var i=0; i<searchProviderLen; i++){
			translatedSearchProviderArray[i] = this._translateCSSClassName(this._selectedSearchProviders[i]);
		}
		return translatedSearchProviderArray;
	},
	
	onSearchProviderSelect: function(selectedMenuItem, event){
		if(selectedMenuItem.get('type') == 'close'){
			dijit.popup.close(selectedMenuItem.getParent());
			return;
		}
		/*String*/
		var itemClass = selectedMenuItem.get('iconClass');
		var untoggledClassName = this._getUntoggledClass(itemClass);
		/*Boolean*/
		var wasToggled = this._isToggledClass(itemClass);
        if(wasToggled){
        	if(this._selectedSearchProviders.length > 1){
        		this._removeSelectedSearchProvider(untoggledClassName);
        	}
        	else{
        		return;
        	}
        }else{
        	this._addSelectedSearchProvider(untoggledClassName);
        }
		console.log(itemClass + " is " + (wasToggled == false ? "no longer " : "now ") + " toggled");
        var newClassName = wasToggled ? untoggledClassName : this._getToggledClass(itemClass);
        console.log("setting new itemClass to "+newClassName);
        selectedMenuItem.set('iconClass',newClassName);
        var sp = this.getSearchProviders();
        console.log(sp);
        //dojo.stopEvent(event);
    },
    
    onChangeCredentials:function(credentialsURL){
    	win.doc.location.href=credentialsURL;
    },
    
    onSignOut:function(){
    	win.doc.location.href="/webstudio/search/logout";
    },
	/**
	 * @return {Handle} dojo handle
	 */
	dojoConnect: function(target /*Object*/, eventName /*String*/, thisObj /*Object*/, handler /*String or Function */){
		return dojo.connect(target, eventName, thisObj ? thisObj : this, handler);
	},
	
	dojoDisconnectByHandle: function (handle /* dojo Handle */){
		dojo.disconnect(handle);
	},
	
	dojoDisconnect: function(/*String*/ handleName){
		if(_handles[handleName]){
			console.log("disconnecting handle with key "+handleName);
			dojo.disconnect(this.getHandle(handleName));
			return true;
		}
		return false;
	},
	submitSearch:function(searchForm /*reference to the DOM form element*/){
		var okToSubmit =  searchForm['q'].value.length > 0;
        if(okToSubmit){
            // make sure there are no duplicate form elements
            $(".__providers").forEach(dojo.destroy);
            dojo.create("input", {type:"hidden", "class":"__providers", name:"providers", value:this.getSearchProviders()},searchForm);
        }
        return okToSubmit;
    },
	toggleMoreLess:function(/*String*/whichSearchProvider, /*Number*/ resultCount, /*Number*/ maxResults, /*Number*/ maxInitialResults ){
		var providerBlockNode = $('#__more'+whichSearchProvider)[0];
		if(!providerBlockNode){
			return;
		}
		var curStyle = domStyle.get(providerBlockNode, "display");
		var moreLinkNode = $('#__summaryHeader'+whichSearchProvider+' .showMore')[0];
		var countSpanNode = $('#__summaryHeader'+whichSearchProvider+' .summaryCountSpan')[0]; 
		if(curStyle != "block"){
			domStyle.set(providerBlockNode,"display", "block");
			domStyle.set(providerBlockNode,"opacity",0);
			fx.fadeIn({node:providerBlockNode}).play();
			var resultCountText = "";
			if(resultCount > maxResults){
				resultCountText = "Top "+maxResults;
			}else{
				resultCountText = resultCount+"";
			}
			countSpanNode.innerHTML = resultCountText; 
			moreLinkNode.innerHTML = resources.SHOW_LESS;
			domAttr.set(moreLinkNode, "title", resources.SHOW_LESS_TITLE);
			
		}else{
			var resultCountText = "";
			if(resultCount > maxInitialResults){
				resultCountText = "Top "+maxInitialResults;
			}else{
				resultCountText = resultCount+"";
			}
			fx.fadeOut({node:providerBlockNode}).play().onEnd = function(evt){domStyle.set(providerBlockNode,"display", "none");};
			countSpanNode.innerHTML = resultCountText; 
			moreLinkNode.innerHTML = resources.SHOW_MORE;
			domAttr.set(moreLinkNode, "title", resources.SHOW_MORE_TITLE);
		}
	},
	onSubmitMouseOver:function(){
		$("#searchButtonUnderlay").style("opacity", 1);
	},
	onSubmitMouseOut:function(){
		$("#searchButtonUnderlay").style("opacity", 0);
	},
	onDropDownMouseOver:function(dropdown){
		var mb = domGeom.getMarginBox(dropdown.domNode);
		$("#dropdownUnderlay").style("width", (mb.w+mb.l)+"px");
		$("#dropdownUnderlay").style("opacity", 1);
	},
	onDropDownMouseOut:function(dropdown){
		$("#dropdownUnderlay").style("opacity", 0);
	},
	doesBrowserSupportOpenSearch:function(){
		if(has("ie") >= 7){
        	return true;
        }
        if(has("ff") >= 3.6){
        	return true;
        }
        if(has("chrome") >= 13){
        	return true;
        }
        return false;
	},
	
    //------------------------------------------------------------------------------------------
    //------------------------"private"---------------------------------------------------------
	//------------------------------------------------------------------------------------------
    
	
	_isToggledClass:function(className){
		if(className == null){
			return;
		}
		var reg = new RegExp("^[^\\s]+ "+this.CONST._toggledClassSuffix+"$");
		return reg.test(className);
	},
	_getToggledClass:function(className){
		if(this._isToggledClass(className)){
			return className;
		}
		return className+" "+ this.CONST._toggledClassSuffix;
	},
	_getUntoggledClass:function(className){
		if(this._isToggledClass(className) == false){
			return className;
		}
		var reg = new RegExp("^([^\\s]+) "+this.CONST._toggledClassSuffix+"$");
		var matches = reg.exec(className);
		if(matches.length > 1){
			return matches[1];
		}
		return null;
	},
	_addSelectedSearchProvider:function(whichClassName){
		if(dojo.indexOf(this._selectedSearchProviders,whichClassName) > -1){
			return false;
		}
		this._selectedSearchProviders.push(whichClassName);
		return true;
	},
	_removeSelectedSearchProvider:function(whichClassName){
		var curIndex =dojo.indexOf(this._selectedSearchProviders,whichClassName);
		if(curIndex < 0){
			return false;
		}
		this._selectedSearchProviders.splice(curIndex, 1);
		return true;
	},
	_translateCSSClassName:function(whichClassName){
		for(var uiName in this._searchProviderProperties){
			if(this._searchProviderProperties[uiName]['cssClass'] == whichClassName){
				return uiName;
			}
		}
		return whichClassName;
	},
	_onAfterSearchHandler:function(){
		var dropdown = dijit.byId('dropDown');
     	dropdown.set("label", this._generateHTMLIconList());
     	var counter = this._selectedSearchProviders.length;
     	if(counter > 0){
     		// the icons that appear on the dropdown button are assigned to the dropdownIcon class
     		// since the icons are being inserted directly into the html we need to wait until they are
     		// loaded before attempting to measure them to resize the input box
	     	$(".dropdownIcon").onload(dojo.hitch(this,function(e){
     			if(--counter <= 0){
     				this._resizeInput();
     			}
     		}));
     	}else{
     		// nothing to load, just do the resize
     		this._resizeInput();
     	}
    },
	_resizeInput:function(){
		var dropdown = dijit.byId('dropDown');
		// use dojo.marginBox here because we want to include the margin in the calculation
		var mb = dojo.marginBox(dropdown.domNode);
     	console.log(mb.w);
     	var searchInput = dojo.byId("searchInput");
     	var newLeftMargin = mb.w + 5; 
     	dojo.style(searchInput, "marginLeft",newLeftMargin+"px");
     	//dojo.position returns the box representing the client area (as opposed to dojo.marginBox)
     	var pmb = dojo.position(searchInput.parentNode);
     	var searchButton = dojo.byId("searchButton");
     	var searchButtonMB = dojo.marginBox(searchButton);
     	console.log(searchButtonMB);
     	console.log(pmb.w);
     	// 5px for padding between the edge of the input box and the search button
     	var searchInputWidth = searchButtonMB.l - newLeftMargin - dojo.style(searchInput.parentNode, "paddingRight") - 5;
     	console.log(searchInputWidth);
     	dojo.style(searchInput, "width",searchInputWidth+"px");
	},
	_typeToIconPath:function(typeStr){
		if(this._searchProviderProperties[typeStr]){
			return this._searchProviderProperties[typeStr]["icon"];
		}
		return "";	
	},
	_generateHTMLIconList:function(){
		var html = "";
		var searchProviders = this.getSearchProviders();
		var searchProvidersLen = searchProviders.length;
		for(var i=0; i<searchProvidersLen; i++){
			var imgFilePath = this._typeToIconPath(searchProviders[i]);
			var props = this._searchProviderProperties[searchProviders[i]];
			html+=this.format("<img src=\"{0}\" title=\"{1} {2}\" class=\"dropdownIcon\"/>", imgFilePath, resources.SEARCH, props.name);
		}
		console.log(html);
		return html;
	},
	format:function(str, replacement0 /* ... */){
		if(arguments.length < 2){
			return str;
		}
		var args = arguments;
		var argLen = args.length;
		var f = function(match, num){
			console.log(num);
			var numArg = parseInt(num);
			if(!isNaN(numArg) && argLen-1 > numArg){
				return args[(numArg+1)];
			}
			return match;
		};
		var reg = /{(\d+)}/gi;
		var newStr = str.replace(reg, f);
		return newStr;
	},
	generateRandomName:function(length){
		var randomName = "";
		if(isNaN(length) || length < 1){
			length = 8;
		}
		var min = 48;
		var max = 122;
		var range = max - min;
		var i = 0;
		while(i < length){
			var ord = Math.round(Math.random() * range + min);
			if( (ord >= 58 && ord <= 64) || (ord >= 91 && ord <=96)){
				// if ord is in these ranges, it's an invalid character.
				continue;
			}
			randomName+=String.fromCharCode(ord);
			i++;
		}
		return randomName;
	},
	_bindSetter:function(/*Function*/ setter, /*Object*/ host, /*String*/ boundFunctionName ){
		var randomName = this.generateRandomName();
		this.setHandle(randomName, this.dojoConnect(host, boundFunctionName, null, setter));
		console.log("handle stored with key "+randomName);
		return randomName;
	},
	_unbindSetter:function(/*String*/ handleName){
		this.dojoDisconnect(handleName);
	}
    
  });
  connect.publish("searchControllerReady");
});